import sympy as sp

from angle_tompy.angle import Angle

from src.math_tompy.symbolic_point2d import revolve


def test_revolve_point_at_basis_success():
    # Setup
    point0: sp.Point2D = sp.Point2D(0, 0)
    point1: sp.Point2D = sp.Point2D(0, 0)
    point2: sp.Point2D = sp.Point2D(0, 0)
    angle0: Angle = Angle(degree=123)

    # Execution
    point3: sp.Point2D = revolve(point=point0, angle=angle0, basis=point1)

    # Validation
    assert point2 == point3


def test_revolve_zero_angle_success():
    # Setup
    point0: sp.Point2D = sp.Point2D(1, 1)
    point1: sp.Point2D = sp.Point2D(0, 0)
    point2: sp.Point2D = sp.Point2D(1, 1)
    angle0: Angle = Angle(degree=0)

    # Execution
    point3: sp.Point2D = revolve(point=point0, angle=angle0, basis=point1)

    # Validation
    assert point2 == point3


def test_revolve_q1_to_q4_success():
    # Setup
    point0: sp.Point2D = sp.Point2D(2, 2)
    point1: sp.Point2D = sp.Point2D(0, 0)
    point2: sp.Point2D = sp.Point2D(2, -2)
    angle0: Angle = Angle(degree=-90)

    # Execution
    point3: sp.Point2D = revolve(point=point0, angle=angle0, basis=point1)

    # Validation
    assert point2 == point3


def test_revolve_q2_to_q4_success():
    # Setup
    point0: sp.Point2D = sp.Point2D(-3, 3)
    point1: sp.Point2D = sp.Point2D(0, 0)
    point2: sp.Point2D = sp.Point2D(3, -3)
    angle0: Angle = Angle(degree=180)

    # Execution
    point3: sp.Point2D = revolve(point=point0, angle=angle0, basis=point1)

    # Validation
    assert point2 == point3



import sympy as sp

from angle_tompy.angle import Angle

from src.math_tompy.symbolic_point2d import sweep_between


def test_vector2_sweep_between_basis_none_success():
    # Setup
    angle0: Angle = Angle(degree=180)
    vector0: sp.Point2D = sp.Point2D(4, 1)
    vector1: sp.Point2D = sp.Point2D(-4, -1)

    # Execution
    angle1: Angle = sweep_between(first=vector0, second=vector1)

    # Validation
    assert angle0 == angle1


def test_vector2_sweep_between_q1_q2_basis_none_success():
    # Setup
    angle0: Angle = Angle(degree=90)
    vector0: sp.Point2D = sp.Point2D(1, 1)
    vector1: sp.Point2D = sp.Point2D(-1, 1)

    # Execution
    angle1: Angle = sweep_between(first=vector0, second=vector1)

    # Validation
    assert angle0 == angle1


def test_vector2_sweep_between_q2_q3_basis_none_success():
    # Setup
    angle0: Angle = Angle(degree=-90)
    vector0: sp.Point2D = sp.Point2D(-1, -1)
    vector1: sp.Point2D = sp.Point2D(-1, 1)

    # Execution
    angle1: Angle = sweep_between(first=vector0, second=vector1)

    # Validation
    assert angle0 == angle1


def test_vector2_sweep_between_q3_q4_basis_none_success():
    # Setup
    angle0: Angle = Angle(degree=90)
    vector0: sp.Point2D = sp.Point2D(-1, -1)
    vector1: sp.Point2D = sp.Point2D(1, -1)

    # Execution
    angle1: Angle = sweep_between(first=vector0, second=vector1)

    # Validation
    assert angle0 == angle1


def test_vector2_sweep_between_q4_q1_basis_none_success():
    # Setup
    angle0: Angle = Angle(degree=90)
    vector0: sp.Point2D = sp.Point2D(1, -1)
    vector1: sp.Point2D = sp.Point2D(1, 1)

    # Execution
    angle1: Angle = sweep_between(first=vector0, second=vector1)

    # Validation
    assert angle0 == angle1


def test_vector2_sweep_between_basis_middle_success():
    # Setup
    angle0: Angle = Angle(degree=180)
    vector0: sp.Point2D = sp.Point2D(4, 5)
    vector1: sp.Point2D = sp.Point2D(-6, 5)
    basis0: sp.Point2D = sp.Point2D(-1, 5)

    # Execution
    angle1: Angle = sweep_between(first=vector0, second=vector1, basis=basis0)

    # Validation
    assert angle0 == angle1



from decimal import Decimal

import pytest

from src.math_tompy.decimal import factorial


def test_factorial_one_success():
    # Setup
    value0: Decimal = Decimal("1")
    factorial0: Decimal = Decimal("1")

    # Execution
    factorial1: Decimal = factorial(value=value0)

    # Validation
    assert factorial0 == factorial1


def test_factorial_two_success():
    # Setup
    value0: Decimal = Decimal("2")
    factorial0: Decimal = Decimal("2")

    # Execution
    factorial1: Decimal = factorial(value=value0)

    # Validation
    assert factorial0 == factorial1


def test_factorial_seven_success():
    # Setup
    value0: Decimal = Decimal("7")
    factorial0: Decimal = Decimal("5040")

    # Execution
    factorial1: Decimal = factorial(value=value0)

    # Validation
    assert factorial0 == factorial1


def test_factorial_twenty_success():
    # Setup
    value0: Decimal = Decimal("20")
    factorial0: Decimal = Decimal("2432902008176640000")

    # Execution
    factorial1: Decimal = factorial(value=value0)

    # Validation
    assert factorial0 == factorial1


def test_factorial_zero_success():
    # Setup
    value0: Decimal = Decimal("0")
    factorial0: Decimal = Decimal("1")

    # Execution
    factorial1: Decimal = factorial(value=value0)

    # Validation
    assert factorial0 == factorial1


def test_factorial_minus_two_success():
    # Setup
    value0: Decimal = Decimal("-2")

    # Validation
    with pytest.raises(ValueError):
        _: Decimal = factorial(value=value0)


def test_factorial_non_whole_number_success():
    # Setup
    value0: Decimal = Decimal("9.738")
    factorial0: Decimal = Decimal("362880")

    # Execution
    factorial1: Decimal = factorial(value=value0)

    # Validation
    assert factorial0 == factorial1
